import os
import collections

import discord
from discord.ext import commands
intents = discord.Intents.all()
bot = commands.Bot(command_prefix='!', intents=intents, case_insensitive=True)

import pickle
import os.path
from googleapiclient.discovery import build
from google_auth_oauthlib.flow import InstalledAppFlow
from google.auth.transport.requests import Request
creds = None
if os.path.exists('token.pickle'):
	with open('token.pickle', 'rb') as token:
		creds = pickle.load(token)
if not creds or not creds.valid:
	if creds and creds.expired and creds.refresh_token:
		creds.refresh(Request())
	else:
		flow = InstalledAppFlow.from_client_secrets_file('credentials.json', SCOPES)
		creds = flow.run_local_server(port=0)
	with open('token.pickle', 'wb') as token:
		pickle.dump(creds, token)
service = build('sheets', 'v4', credentials=creds)
sheet = service.spreadsheets()


from dotenv import load_dotenv
load_dotenv()

from . import logging
logger = logging.logger

import discord_argparse
from . import parser
bot.help_command = parser.MyHelpCommand()

from .message_handler import PhraseReponse
from .event_data import *


#: response emoji when multiple options
SELECTION_EMOJIS = collections.OrderedDict(
	[
		("1️⃣", 1),
		("2️⃣", 2),
		("3️⃣", 3),
		("4️⃣", 4),
		("5️⃣", 5),
		("6️⃣", 6),
		("7️⃣", 7),
		("8️⃣", 8),
		("9️⃣", 9),
		("🔟", 10),
	]
)

dicMessage = {
	# General phrases
	'check-error': "❌ You do not have the required Privileges",
	'cmd-disabled': "❌ The command is not active currently",

	# Member voting phrases
	'vote-macrotype': """🤖 **Bip, Boup**
⚠️ I was not able to handle the macrotype you've given me
✉️ Can you react to the right one, please ?
1️⃣ Aggro     2️⃣ Tempo     3️⃣ Control   4️⃣ Combo     5️⃣ Midrange""",
	'vote-error': "❌ The reaction you just used has raised an error. Please reach out to a staff member to get some help",
	'vote-success': "✅ Thank you for your input",

	# Registration phrases
	'registration-complete': "✅ You have been successfully registered !",
	'registration-registered': "❌ You are already registered, please use this command only once",
	'registration-full': "🤖 **Bip, Boup**\nThe event is full, I'll check if I can add you to the waiting list",
	'registration-ok': "📝 **New Registration**\nMaster Table has been updated\nMessage content: ",
	'registration-missing': "❌ There are some required information that you did not give us. Please refer to `!help register` for the full command and details about the arguments",
	'registration-disabled': "❌ Registration are closed for all of our events for now",
	'registration-bad-argument': "❌ The arguments used do not fit with what I've been programmed for. Please reach out to a staff member to get som help",

	# Registration management phrases
	'open-success': "🟢 Registrations for the following event are activated: ",
	'open-pb-argument': "❌ The argument used do not fit main, side or fun event",
	'close-event': "🟢 Registrations for the following event are disabled: ",
}


def getLeague():
	"""Get the Guild object"""
	return bot.get_guild(int(LIGUEID))

def getChannel(id):
	"""Getting the Channel object"""
	for channel in getLeague().channels:
		if channel.id == id:
			return channel

def getMember(ctx):
	"""Get member if message.author is a guild.member"""
	for member in getLeague().members:
		if member.id == ctx.author.id:
			return member

def getTable(range):
	"""GSuite procedure to manage Spreadsheets"""
	return sheet.values().get(spreadsheetId=SHEETID,range=range).execute().get('values')

def getEvent(key: str):
	table = {
		'Main Event': MAINEVENT,
		'Side Event': SIDEEVENT,
		'Fun Event' : FUNEVENT,
	}
	return table[key]


def is_admin(ctx):
	for role in getMember(ctx).roles:
		if role.permissions.administrator:
			return True
	return False



async def memberVote(member, message: str, options: int):
	"""Method to ask more information from user using reactions"""
	message = await sendMessage(member, message)
	try:
		# try loop in case adding reaction is Discord.Forbidden
		for reaction in list(SELECTION_EMOJIS.keys())[: options]:
			await message.add_reaction(reaction)
	except discord.Forbidden:
		logger.warning("Missing reaction permission")

	def check(reaction, user):
		return user == member and str(reaction.emoji) in SELECTION_EMOJIS and reaction.count == 2
	reaction, user = await bot.wait_for('reaction_add', check=check)

	#Recuperation du message envoyé et de la réaction en double associé
	message = await member.fetch_message(message.id)
	for reaction in message.reactions:
		if reaction.count == 2:
			await sendMessage(member, dicMessage['vote-success'])
			return SELECTION_EMOJIS[str(reaction.emoji)]

	# If a reaction is misinterpreted
	await sendMessage(member,dicMessage['vote-error'])


async def sendMessage(ctx,str):
	"""Method to send message"""
	if str == "":
		return
	while len(str) > 2000 :
		prov = str[:2000]
		i=0
		for x in prov :
			i=i+1
			if x == "\n" :
				if "\n" not in prov[i:] :
					await ctx.send(prov[:i])
					str = str[i:]
	else:
		return await ctx.send(str)

async def deleteMessage(message):
	"""Method to delete a message"""
	try:
		await message.delete()
	except:
		return

def setTable(range,values): #Procedure G.sheet pour editer le tableau
	body = {'values': values}
	send = sheet.values().update(
			spreadsheetId=SHEETID,
			range=range,
			valueInputOption='USER_ENTERED',
			body=body
		).execute()

def registrering (event, list, member, nickname, hash, link, macrotype: str = "0"):
	"""Preparing new version of the table"""
	table = getTable(event[list])
	for line in table:
		if line[0] == str(member.id):
			return { 'status': 'error', 'msg': 'registration-registered' }
		if line[0] == "":
			line[0] = str(member.id)
			line[1] = nickname
			line[2] = hash
			line[3] = link
			line[4] = macrotype
			setTable(event[list], table)
			return { 'status': 'success', 'msg': 'registration-complete' }
	return { 'status': 'error', 'msg': 'registration-full' }



@bot.listen()
async def on_ready():
	"""Login success informative log"""
	logger.info("Logged in as {}", bot.user)
	await bot.change_presence(activity=discord.Game(name="Duel Commander"))


@bot.command(
	name="open",
	help="Open registration for said event (either main, side or fun)",
	hidden=True,
)
@commands.check(is_admin)
async def openRegistration(ctx, arg:str):
	logger.info("Received command: {m}".format(m=ctx.message.content))
	events = { 'mai': 'Main Event', 'sid': 'Side Event', 'fun': 'Fun Event' }
	if arg.lower()[:3] in events.keys():
		await deleteMessage(ctx.message)
		await sendMessage(
			getChannel(BOTRETOURGENERAL),
			dicMessage['open-success']+"**"+events[arg.lower()[:3]]+"**"
		)
		registration.update(enabled=True)
		closeRegistration.update(enabled=True)
		return openRegistration.update(enabled=False,aliases=[events[arg.lower()[:3]]])
	raise discord.ext.commands.errors.BadArgument

@openRegistration.error
async def openRegistration_error(ctx, error):
	await deleteMessage(ctx.message)
	if isinstance(error, discord.ext.commands.errors.BadArgument):
		return await sendMessage(getChannel(BOTRETOURGENERAL),dicMessage['open-pb-argument'])
	if isinstance(error, discord.ext.commands.errors.MissingRequiredArgument):
		return await sendMessage(getChannel(BOTRETOURGENERAL),dicMessage['open-pb-argument'])
	if isinstance(error, discord.ext.commands.DisabledCommand):
		return await sendMessage(ctx.author,dicMessage['cmd-disabled'])
	if isinstance(error, discord.ext.commands.CheckFailure):
		return await sendMessage(ctx.author,dicMessage['check-error'])
	return await sendMessage(
		getChannel(BOTRETOURGENERAL),
		(
			f"❌ Error during command handling for **{ctx.message.author}**\nMessage content:"
			+ f"```{ctx.message.content}```"
		)
	)


@bot.command(
	name="close",
	help="Close registration for said event (either main, side or fun)",
	hidden=True,
	enabled=False,
)
@commands.check(is_admin)
async def closeRegistration(ctx):
	logger.info("Received command: {m}".format(m=ctx.message.content))
	await deleteMessage(ctx.message)
	event = openRegistration.aliases[0]

	openRegistration.update(aliases=[],enabled=True)
	closeRegistration.update(enabled=False)
	registration.update(enabled=False)

	return await sendMessage(
		getChannel(BOTRETOURGENERAL),
		dicMessage['close-event']+"**"+event+"**"
	)

@closeRegistration.error
async def closeRegistration_error(ctx, error):
	await deleteMessage(ctx.message)
	if isinstance(error, discord.ext.commands.DisabledCommand):
		return await sendMessage(ctx.author,dicMessage['cmd-disabled'])
	if isinstance(error, discord.ext.commands.CheckFailure):
		return await sendMessage(ctx.author,dicMessage['check-error'])
	return await sendMessage(
		getChannel(BOTRETOURGENERAL),
		(
			f"❌ Error during command handling for **{ctx.message.author}**\nMessage content:"
			+ f"```{ctx.message.content}```"
		)
	)

@bot.command(
	name="register",
	aliases=["inscription"],
	brief="Register a player to an event",
	help="Register a player into an open-for-registration tournament with a Cockatrice nickname, the deck's hash from Cockatrice and a Moxfield link to the decklist",
	usage="nickname=player_nickname hash=deck_hash link=deck_link",
	enabled=False,
)
async def registration(ctx, *, args: parser.registration=parser.registration.defaults()):
	logger.info("Received command: {m}".format(m=ctx.message.content))
	await deleteMessage(ctx.message)

	event = getEvent(openRegistration.aliases[0])
	member = getMember(ctx)

	"""
	Registering without macrotype
	Checking this information after registration as it is not a RequiredArgument
	"""
	reg = registrering(
		event, 'ListeInscrit',
		member, args["nickname"], args["hash"], args["link"], ""
	)
	await sendMessage(member, dicMessage[reg['msg']])
	# automatically registered if a seat is available

	wl = {'status':'','msg':''}
	if reg['msg'] == 'registration-full':
		wl = registrering(
			event, 'ListedAttente',
			member, args["nickname"], args["hash"], args["link"], ""
		)
		await sendMessage(member, dicMessage[wl['msg']])
	if reg['msg'] == 'registration-complete' or wl['msg'] == 'registration-complete':
		for role in getLeague().roles:
			if role.name == event["NomRole"]:
				await member.add_roles(role)
	if reg['msg'] == 'registration-registered' or wl['msg'] == 'registration-registered':
		return

	"""
	Checking and updating macrotype
	"""
	macrotypes = ["aggro", "tempo", "control", "combo", "midrange", "agro", "controle", "midrang"]
	if args["macrotype"] != "" and args["macrotype"].lower() not in macrotypes:
		macrotype = await memberVote(member, dicMessage['vote-macrotype'], 5)
		args["macrotype"] = macrotypes[macrotype-1]
	if reg['msg'] == 'registration-complete':
		list = 'ListeInscrit'
	elif wl['msg'] == 'registration-complete':
		list = 'ListedAttente'
	table = getTable(event[list])
	for line in table:
		if line[0] == str(member.id):
			line[4] = args["macrotype"]
			setTable(event[list], table)
			break

	"""
	Sending Log messages
	"""
	await sendMessage(
		getChannel(event["bot-channel"]),
		dicMessage['registration-ok']+"`"+ctx.message.content+"`\nUser: **"+args['nickname']+"**"
	)

@registration.error
async def registration_error(ctx, error):
	await deleteMessage(ctx.message)
	if isinstance(error, discord.ext.commands.errors.CommandInvokeError):
		await sendMessage(ctx.author,dicMessage['registration-missing'])
	if isinstance(error, discord.ext.commands.DisabledCommand):
		return await sendMessage(ctx.author,dicMessage['registration-disabled'])
	if isinstance(error, discord.ext.commands.errors.BadArgument):
		await sendMessage(ctx.author,dicMessage['registration-bad-argument'])
	return await sendMessage(
		getChannel(BOTRETOURGENERAL),
		(
			f"❌ Error during registration for **{ctx.message.author}**\nMessage content:"
			+ f"```{ctx.message.content}```"
		)
	)

def main():
	"""Entrypoint for the Discord Bot"""
	logger.setLevel(logging.INFO)
	bot.run(os.getenv("DISCORD_TOKEN"))
	# reset log level so as to not mess up tests
	logger.setLevel(logging.NOTSET)



import collections
import json

from discord.ext import commands
from discord_argparse import *

class MyHelpCommand(commands.DefaultHelpCommand):
	async def send_command_help(self, command):
		self.add_command_formatting(command)
		for name, param in command.clean_params.items():
			if isinstance(param.annotation, ArgumentConverter):
				arguments = param.annotation.arguments
				if not arguments:
					continue
				self.paginator.add_line("Arguments:")
				max_size = max(len(name) for name in arguments)

				for name, argument in arguments.items():
					entry = "{0}{1:<{width}} {2}".format(self.indent * " ", name, argument.doc, width=max_size)
					self.paginator.add_line(self.shorten_text(entry))
		self.paginator.close_page()
		await self.send_pages()

class ArgumentConverter(ArgumentConverter):
	def sort(self):
		self.arguments = collections.OrderedDict(sorted(self.arguments.items()))

	def get_args(self):
		return self.arguments.keys()

	def __add__(self, other):
		args = {}
		for k,v in self.arguments.items():
			args[k] = v
		for k,v in other.arguments.items():
			if k in args.keys():
				pass
			args[k] = v
		return ArgumentConverter(**args)




# Registration parser
registration = ArgumentConverter(
	nickname = RequiredArgument(
		str,
		doc="Nickname on Rooster Rangers' Cockatrice Server"
	),
	hash = RequiredArgument(
		str,
		doc="Hashcode from Cockatrice's deckfile"
	),
	link = RequiredArgument(
		str,
		doc="Moxfield link to the decklist"
	),
	macrotype = OptionalArgument(
		str,
		doc="Macrotype of the deck: Aggro, Tempo, Control, Combo, Midrange",
		default=""
	)
)



SCOPES = ['https://www.googleapis.com/auth/spreadsheets']
LIGUEID = 817430786629173258
SHEETID = "1lJTFm6xMdUYA22tadIdAMZO5pYOH3Sc1OEVZsfeB_h4"
STAT = "Split_2!A4:S53"
BOTSPAM = 833778930858852413
BOTRETOURGENERAL = 834892751363375155
MAINEVENT1 = {
	"ListeMatch": "S3_ME!V4:Z170",
	"ListeScore": "S3_ME!W4:X170",
	"ListeInscrit": "S3_ME!P4:U51",
	"ListedAttente": "S3_ME!P52:U170",
	"bot-channel": 848664401069867088,
	"Categorie": 702453004179079248,
	"NomRole": "MainEvent"
}
SIDEEVENT1 = {
	"ListeMatch": "S3_SE!V4:Z110",
	"ListeScore": "S3_SE!W4:X110",
	"ListeInscrit": "S3_SE!P4:U27",
	"ListedAttente": "S3_SE!P28:U110",
	"bot-channel": 848664537355386890,
	"Categorie": 805195339803131914,
	"NomRole": "SideEvent"
}
FUNEVENT1 = {
	"ListeMatch": "S3_FE!V4:Z66",
	"ListeScore": "S3_FE!W4:X66",
	"ListeInscrit": "S3_FE!P4:U23",
	"ListedAttente": "S3_FE!P24:U66",
	"bot-channel": 848664935332446228,
	"Categorie": 810987751117815838,
	"NomRole": "FunEvent"
}
MAINEVENT2 = {
	"ListeMatch": "S4_ME!V4:Z170",
	"ListeScore": "S4_ME!W4:X170",
	"ListeInscrit": "S4_ME!P4:U51",
	"ListedAttente": "S4_ME!P52:U170",
	"bot-channel": 848664401069867088,
	"Categorie": 702453004179079248,
	"NomRole": "MainEvent"
}
SIDEEVENT2 = {
	"ListeMatch": "S4_SE!V4:Z110",
	"ListeScore": "S4_SE!W4:X110",
	"ListeInscrit": "S4_SE!P4:U27",
	"ListedAttente": "S4_SE!P28:U110",
	"bot-channel": 848664537355386890,
	"Categorie": 805195339803131914,
	"NomRole": "SideEvent"
}
FUNEVENT2 = {
	"ListeMatch": "S4_FE!V4:Z66",
	"ListeScore": "S4_FE!W4:X66",
	"ListeInscrit": "S4_FE!P4:U23",
	"ListedAttente": "S4_FE!P24:U66",
	"bot-channel": 848664935332446228,
	"Categorie": 810987751117815838,
	"NomRole": "FunEvent"
}
MAINEVENT = MAINEVENT2
SIDEEVENT = SIDEEVENT2
FUNEVENT = FUNEVENT2



def PhraseReponse(arg): #Dictionnaire de réponse du bot
	if arg == "Quel Event ?":
		msg = "Je n'arrive pas à determiner dans quel event tu as fait ta commande. Peut-tu m'aider en votant ci dessous ?"
		msg = msg + "\n:one: pour MainEvent ou :two: pour SideEvent ou :three: pour FunEvent"
		return msg
	if arg == "Erreur Check":
		return "❌ Tu n'as pas les droits pour utiliser cette commande"
	if arg == "Erreur Vote":
		return "❌ Contact un administrateur car je n'ai pas compris ton vote"

	if arg[0] == "Affichage":
		if arg[1] == "Argument Invalide":
			return "❌ Les paramètres que tu as entré sont invalides, tu as mis des valeurs trop grandes, trop petites ou une lettre qui ne correspond pas à une poule"

	if arg[0] == "Ouverture/Fermeture":
		if arg[1] == "Inscriptions ouvertes":
			return f"🟢 Les inscriptions pour le {arg[2]} sont ouvertes"
		if arg[1] == "Inscriptions fermees":
			return f"🔴 Les inscriptions pour le {arg[2]} sont fermees"
		if arg[1] == "Commande inactive":
			return "❌ La commande est inactive pour le moment"
		if arg[1] == "Argument Manquant":
			return "❌ Il faut préciser quel evenement est ouvert aux inscriptions ? Main, Side ou Fun ?"
		if arg[1] == "Argument Invalide":
			return "❌ Je n'ai pas compris quel evenement est ouvert aux inscriptions ? Main, Side ou Fun ?"

	if arg[0] == "Inscription":
		if arg[1] == "Validé":
			return "✅ Ton inscription a bien été validé !"
		if arg[1] == "Tournoi Plein":
			return "\nLe tournoi est plein, tu es dans une liste d'attente"
		if arg[1] == "Quel Macro ?":
			msg = "**Macrotype inconnu**"
			msg = msg + "\nTu n'as pas rajouter de macrotype lors de ton inscription ou bien ce dernier est invalide."
			msg = msg + "\nNous nous basons sur le classement **C** disponible sur le Barrin's Codex."
			msg = msg + "\nhttps://barrins-codex.org/fr/articles/classifier-un-deck/les-macrotypes.html#5_macrotypes"
			msg = msg + "\nTu peux voter ici pour rajouter ton macrotype"
			return msg + "\n:one: pour Aggro ou :two: pour Combo ou :three: pour Control ou :four: pour Midrange ou :five: pour Tempo"
		if arg[1] == "Ajout macro":
			return "✅ Merci d'avoir rajouter ton macrotype, tu as voté : "
		if arg[1] == "Deja inscrit":
			return "❌ Tu es déjà inscrit ! On ne s'inscrit qu'une seule fois !!"
		if arg[1] == "Commande inactive":
			return "❌ Les inscriptions sont fermées pour le moment"
		if arg[1] == "Argument Manquant":
			return "❌ Il faut que je connaisse, votre pseudo Cockatrice, le hastag de votre deck, un lien internet vers votre decklist ainsi que votre eventuel macrotype, ni plus, ni moins."
		if arg[1] == "Argument Invalide":
			return "❌ Je n'ai pas compris les parametres de votre inscriptions, veuillez contacter un administrateur"

	if arg[0] == "Resultat":
		if arg[1] == "Eliminé":
			return "🔴 Tu n'as plus de match à faire dans le tournoi, merci d'avoir participé"
		if arg[1] == "Attente":
			return "🟠 On attend que ton prochain adversaire soit décidé, tu recevra un message lorsqu'on le connaitra"
		if arg[1] == "Next Match vs : ":
			return f"🟢 Ton prochain match est prêt, tu va devoir affronter : **{arg[2]}**"
		if arg[1] == "Joueur introuvable":
			return f"❌ **{arg[2]}** est introuvable dans le tournoi(pas inscrit) ou dans le serveur(absent)."
		if arg[1] == "Auteur invalide":
			return "❌ Tu ne fais pas parti des joueurs du match... tu n'as pas le droit de rentrer le score"
		if arg[1] == "Score Impossible":
			return "❌ Ce résultat me semble étrange, pourrais-tu vérifier stp ?"
		if arg[1] == "Match Introuvable":
			return "❌ Le match est introuvable, est-tu sur que ces deux joueurs se rencontrent ?"
		if arg[1] == "Score Existant":
			return "❌ Le score est déjà écrit, si il y a contestation il faut aller voire un administrateur."
		if arg[1] == "Score validé":
			return "✅ Le score a été enregistré :\n"
		if arg[1] == "Argument Manquant":
			return "❌ Il me manque l'un des joueurs ou le Score, as-tu bien repescté les espaces : Joueur1 0-0 Joueur2"
		if arg[1] == "Argument Invalide":
			return "❌ Les paramètres que tu as entré sont invalides il faut écrire sous la forme : Joueur1 0-0 Joueur2"

	if arg[0] == "Statistique":
		if arg[1] == "Joueur introuvable":
			return f"Je ne trouve pas le joueur **{arg[2]}** parmis les participants de cette saison"

	if arg[0] == "Invitationnal":
		msg = "Voici les joueurs actuellement invitées et la listes des matchs prévus pour le prochain TOP8 Invitationnal : \n"
		msg = msg + f"-🎲 1e : {arg[1][0]} VS 8e : {arg[1][7]}\n-🎲 4e : {arg[1][3]} VS 5e : {arg[1][4]}\n-🎲 2e : {arg[1][1]} VS 7e : {arg[1][6]}\n-🎲 3e : {arg[1][2]} VS 6e : {arg[1][5]}\n"
		msg = msg + "**La personne écrite à gauche décidera de commencer ou non la première manche**\n"
		return msg + "Attention, si les saisons en cours ne sont pas terminés ce classement peut encore changer."

	if arg[0] == "PLS":
		return f"❌ La fonction {arg[1].command.name} a planté, les administrateurs ont normalement été mis au courant. Nous cherchons à résoudre ce problème."

	if arg[0] == "Bot":
		return f"Message envoyé depuis <#{arg[1].channel.id}> par **{arg[1].author.name}**:\n*Contenu du message* `{arg[1].message.content}`"
	if arg[0] == "MPBot":
		return f"✉️\nLe BOT a envoyé le message suivant dans les MP de **{arg[1]}**:```{arg[2]}```"
	if arg[0] == "PLSBOT":
		return f"⚠️⚠️⚠️⚠️⚠️\nLa fonction {arg[1].command.name} a planté, voici le contexte : \nMessage envoyé depuis <#{arg[1].channel.id}> par **{arg[1].author.name}**:\n*Contenu du message* `{arg[1].message.content}`"

	# Cas par défaut
	return "⚠️ Je ne sais pas ce que je dois dire"


dicMessage = {
	# Member voting phrases
	'vote-macrotype': """🤖 **Bip, Boup**
⚠️ I was not able to handle the macrotype you've given me
✉️ Can you react to the right one, please ?
1️⃣ Aggro     2️⃣ Tempo     3️⃣ Control   4️⃣ Combo     5️⃣ Midrange""",
	'vote-error': "❌ The reaction you just used has raised an error. Please reach out to a staff member to get some help",
	'vote-success': "Thank you for your input, I have taken appropriate measures",

	# Registration phrases
	'registration-complete': "✅ You have been successfully registered !",
	'registration-registered': "❌ You are already registered, please use this command only once",
	'registration-full': "🤖 **Bip, Boup**\nThe event is full, I'll check if I can add you to the waiting list",
	'registration-ok': "📝 **New Registration**\nMaster Table has been updated\nMessage content: ",
	'registration-missing': "❌ There are some required information that you did not give us. Please refer to `!help register` for the full command and details about the arguments",
	'registration-disabled': "❌ Registration are closed for all of our event for now",
	'registration-bad-argument': "❌ The arguments used do not fit with what I've been programmed for. Please reach out to a staff member to get som help",
}



"""Custom LoggerAdapter and formatter."""
"""Module made by lionel-panhaleux."""
import logging


# forward log levels for convenience
NOTSET = logging.NOTSET
DEBUG = logging.DEBUG
INFO = logging.INFO
WARNING = logging.WARNING
ERROR = logging.ERROR
CRITICAL = logging.CRITICAL


class LineLogAdapter(logging.LoggerAdapter):
	"""Add line number to the log, use {} format strings."""

	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		self.line = None

	def process(self, msg, kwargs):
		"""Accept an extra line argument, but also use the line set on self."""
		self.extra.update(kwargs.get("extra", {}))
		prefix = ""
		if self.extra.get("line"):
			prefix += "[%6s]" % self.extra["line"]
		if self.extra.get("deck"):
			prefix += "[%s] " % self.extra["deck"]
		elif prefix:
			prefix += " "
		return prefix + msg, kwargs

	class Message:
		"""A {}-formatted message"""

		def __init__(self, fmt, *args, **kwargs):
			self.fmt = fmt
			self.args = args
			self.kwargs = kwargs

		def __str__(self):
			return self.fmt.format(*self.args, **self.kwargs)

	def log(self, level, msg, *args, **kwargs):
		"""Add line number if any, format string using the str.format() method."""
		if self.isEnabledFor(level):
			msg, kwargs = self.process(msg, kwargs)
			self.logger._log(level, self.Message(msg, *args, **kwargs), ())


_handler = logging.StreamHandler()
_handler.setFormatter(logging.Formatter("[%(levelname)7s] %(message)s"))
_base_logger = logging.getLogger()
_base_logger.addHandler(_handler)
logger = LineLogAdapter(logging.getLogger("league-bot"), extra={"line": None})
